//---------------------------------------------------------------------------
#pragma hdrstop

#include "AT90S2323.h"          //     AT90S2323
#include "PortFunctions.h"      //      
#include "LogFunctions.h"       //      
#include "ChipFunctions.h"      //     
#include "Main.h"               //      
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---       PROGRAMMING ENABLE -----
//---------------------------------------------------------------------------
bool AT90S2323_ProgrammingEnable()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," execute Programming Enable");               //     Programming Enable
  //---      SS ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x48;                                    // SetSS_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x48");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---      SS ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x47;                                    // ClearSS_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x47");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //---   20  ---
  Sleep(20);
  //---       Programming Enable ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x53;
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x53 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-      ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---  ,    - ,   ,   -   
  if(rd_buf.Data[1]!=0x53) return false;                //    ( 1)
  SendMsgToLog(19," with success.");                    // ,    
  return true;
}
//---------------------------------------------------------------------------
//---    CHIP ERASE -------------------------------------
//---------------------------------------------------------------------------
bool AT90S2323_ChipErase()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," erase the chip");    //     
  //---       ChipErase ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x80;
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x80 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---      18  ---
  Sleep(18);
  //--- ,    ---
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
///////////////////////////////
//     //
///////////////////////////////
//---------------------------------------------------------------------------
//---       ---------------------------
//---------------------------------------------------------------------------
bool AT90S2323_ReadProgramMemory(unsigned int Address, bool Action)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char H;       // ,      
  unsigned char a, b;    //      

  //         
  H =(Address&1)<<3;    //   ,        3- 
  b = Address>>1;       //    
  a = (Address>>9)&3;   //     (,     - )

  //---       ReadProgramMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=0x0;                           //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=a;
  wr_buf.Data[4]=0x20|H;                        //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 b a 0x2H");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---              ---
  if(!Action)   //     (0)
  {     SelectedChip.Area1[Address] = rd_buf.Data[0];
  }
  else          //     (1)
  {     if(SelectedChip.Area1[Address] != rd_buf.Data[0])       //      ,   
        {       return false;           //   
        }
  }
  return true;
}
//---------------------------------------------------------------------------
//---       ---------------------------
//---------------------------------------------------------------------------
bool AT90S2323_WriteProgramMemory(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char H;       // ,      
  unsigned char a, b;    //      

  //         
  H =(Address&1)<<3;    //   ,        3- 
  b = Address>>1;       //    
  a = (Address>>9)&3;   //     (,     - )

  //---       WriteProgramMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=SelectedChip.Area1[Address];   //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=a;
  wr_buf.Data[4]=0x40|H;                        //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 Data b a 0x4H");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(9);
  return true;
}
//---------------------------------------------------------------------------
/////////////////////////////
//    EEPROM //
/////////////////////////////
//---------------------------------------------------------------------------
//---    EEPROM    ---------------------------
//---------------------------------------------------------------------------
bool AT90S2323_ReadEEPROMMemory(unsigned int Address, bool Action)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char b;      //   

  b = Address&0x7F;     //     (   128  EEPROM)

  //---       ReadEEPROMMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=0x0;                           //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=0x0;
  wr_buf.Data[4]=0xA0;                          //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 b 0x00 0xA0");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---               ---
  if(!Action)   //     (0)
  {     SelectedChip.Area2[Address] = rd_buf.Data[0];
  }
  else          //     (1)
  {     if(SelectedChip.Area2[Address] != rd_buf.Data[0])       //      ,   
        {       return false;           //   
        }
  }
  return true;
}
//---------------------------------------------------------------------------
//---     EEPROM    ---------------------
//---------------------------------------------------------------------------
bool AT90S2323_WriteEEPROMMemory(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char b;      //  

  b = Address&0x7F;     //     (   128  EEPROM)

  //---       LoadProgramMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=SelectedChip.Area2[Address];   //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=0x0;
  wr_buf.Data[4]=0xC0;                          //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 Data b 0x00 0xC0");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(9);
  return true;
}
//---------------------------------------------------------------------------
/////////////////////
// FUSES, LOCK ... //
/////////////////////
//---------------------------------------------------------------------------
//---  LOCK-  FUSE- ----------------------------------------
//---------------------------------------------------------------------------
bool AT90S2323_ReadLockFuseBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," read lock- & fuse-bits");    //     Lock-  Fuse-
  //---       ReadLockBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x00;
  wr_buf.Data[4]=0x58;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x00 0x58");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---         lock- ---
  SelectedChip.Area3[0] = rd_buf.Data[0]|0xDE;  // Fuse- (  0-  5-,  = 1)
  // Lock- (7-  -  1-, 6-  -  2-)
  SelectedChip.Area3[1] = 0xFF;
  if(!(rd_buf.Data[0]&0x80))    SelectedChip.Area3[1] &= ~2;     //  7-  -   1
  if(!(rd_buf.Data[0]&0x40))    SelectedChip.Area3[1] &= ~4;     //  6-  -   2
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---  LOCK- -----------------------------------------------------
//---------------------------------------------------------------------------
bool AT90S2323_WriteLockBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write lock-bits");    //     Lock-
  //---       WriteLockBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=SelectedChip.Area3[1];
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 Lock 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(5);
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---  FUSE- (FUSE BYTE) -----------------------------------------
//---------------------------------------------------------------------------
bool AT90S2323_WriteFuseBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write fuse byte");              //      
  //---       WriteFuseBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x0;                                     //    
  wr_buf.Data[2]=0x0;
  wr_buf.Data[3]=0xBF & SelectedChip.Area3[0];            // 0xBF & Fuse-
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 FSTRT 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(5);
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---    -----------------------------------------------
//---------------------------------------------------------------------------
unsigned int AT90S2323_ReadSignatureBytes()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned int ChipID, Temp;    //  

  SendMsgToLog(18," read signature bytes");     //      
  ChipID = 0;                                   //  
  //---       Read Signature Byte ---
  for(unsigned char i=0;i<3;i++)
  {     wr_buf.Size=5;
        wr_buf.Data[0]=0x44;                                 // LoadPaket_SPI
        wr_buf.Data[1]=0x0;
        wr_buf.Data[2]=i;                                    //   
        wr_buf.Data[3]=0x0;
        wr_buf.Data[4]=0x30;                                 //  Read Signature Byte
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 i 0x00 0x30");   //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        wr_buf.Size=1;
        wr_buf.Data[0]=0x46;                                  // StartChange_SPI
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                               CloseHandle(Port);
                                           Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        Sleep(Delays);//////////
        //---  SPI -  (        ) ---
        wr_buf.Size=1;
        wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
        //  ,  SPI - 
        rd_buf=ReadFrom(8);                                   // 
        if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return 0;
                         }
        //---  ID 
        Temp = rd_buf.Data[0];
        ChipID += (Temp << (8*i));
  }
  SendMsgToLog(19,"");            //    
  return ChipID;
}
//---------------------------------------------------------------------------

