//---------------------------------------------------------------------------
#pragma hdrstop

#include "ATtiny2313.h"         //       ATtiny2313
#include "AVRFunctions.h"       //       AVR
#include "PortFunctions.h"      //      
#include "LogFunctions.h"       //      
#include "ChipFunctions.h"      //     
#include "Main.h"               //      
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---       PROGRAMMING ENABLE -----
//---------------------------------------------------------------------------
bool ATtiny2313_ProgrammingEnable()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," enter Programming Mode");    //        
  //---          ---
  //---  RDY       
  //---        (  ,   )
  //---      

  //---   B
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0B;                                                  // DDRBSetup
  wr_buf.Data[1]=0x0;                                                   //    
  wr_buf.Data[2]=0x0D;                                                  // PORTBSetup
  wr_buf.Data[3]=0x0;                                                   //    
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set DDRB=0, PORTB=0");        //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //---   C
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0F;                                                  // DDRCSetup
  wr_buf.Data[1]=0x32;  // 0011 0010                                    //     ,  RDY   
  wr_buf.Data[2]=0x11;                                                  // PORTCSetup
  wr_buf.Data[3]=0x0;                                                   //     (  )
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set DDRC=0x32, PORTC=0x0");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //---   D (     Prog_enable)
  //---   PG   ,        
  wr_buf.Size=4;
  wr_buf.Data[0]=0x13;                                                  // DDRDSetup
  wr_buf.Data[1]=0xFC;  // 1111 1100                                    //     
  wr_buf.Data[2]=0x15;                                                  // PORTDSetup
  wr_buf.Data[3]=0x0;                                                   //     (       WR, OE)
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set DDRD=0xFC, PORTD=0x0");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //---          AVR ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                                  //   
  wr_buf.Data[1]=0x0;                                                   //  Enter_HVP_ProgMode_AVR
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," EnterHVPProgMode_AVR");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //---   D -     WR, OE (      )
  wr_buf.Size=2;
  wr_buf.Data[0]=0x15;                                          // PORTDSetup
  wr_buf.Data[1]=PD_WORK;       // 0001 0100                    //  WR, OE  1
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set WR=1, OE=1");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  SendMsgToLog(19," with success.");                    // ,    
  return true;
}
//---------------------------------------------------------------------------
//---    CHIP ERASE -------------------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_ChipErase()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," erase the chip");                           //     
  if(!PhaseA(0x80))        return 0;                            // Load Command 0x80 - Chip Erase
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                          // Do_Extended_Command
  wr_buf.Data[1]=0x03;                                          // GiveWRNegPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," GiveWRNegPulse");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(Delays);//////////
  //   RDY = High
  if(!WaitRDYHigh)      return 0;               //    -   
  //--- ,    ---
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
///////////////////////////////
//     //
///////////////////////////////
//---------------------------------------------------------------------------
//---       ---------------------------
//---         ---------------------
//---------------------------------------------------------------------------
bool ATtiny2313_ReadProgramMemory(unsigned int Address, bool Action)
{ WRITE_BUFFER wr_buf;  //    
  READ_BUFFER rd_buf;   //    
  unsigned char a, b;   //      

  //        
  b = Address>>1;       //    
  a = Address>>9;       //    
  if(Address == 0)      //    ,   
  {     if(!PhaseA(0x02))       return 0;       // Load Command 0x02 - Read Flash
  }
  if(!PhaseC(a))        return 0;               // Load Address High Byte
  if(!PhaseB(b))        return 0;               // Load Address Low Byte

  //--- set DDRB to input and OE to 0 to read a flash word low byte (BS1 is already 0 after Phase B)
  wr_buf.Size=5;
  wr_buf.Data[0]=0x0B;                                  // DDRBSetup
  wr_buf.Data[1]=0x0;   // 0000 0000                    //    
  wr_buf.Data[2]=0x15;                                  // PORTDSetup
  wr_buf.Data[3]=PD_WORK & ~(1<<OE);    // 0001 0000    //  OE
  wr_buf.Data[4]=0x17;                                  // PINBRead
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," read flash word low byte");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---  ,  PINB
  rd_buf=ReadFrom(1);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");         //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  //---              ---
  if(!Action)   //    
  {     SelectedChip.Area1[Address] = rd_buf.Data[0];
  }
  else          //    
  {     if(SelectedChip.Area1[Address] != rd_buf.Data[0])       //      ,   
        {       return 0;
        }
  }
  //--- set BS1 to 1 to read a flash word high byte, read byte, set OE
  wr_buf.Size=5;
  wr_buf.Data[0]=0x11;                                                  // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1 << BS1);  // 0000 1010                    //  BS1
  wr_buf.Data[2]=0x17;                                                  // PINBRead
  wr_buf.Data[3]=0x15;                                                  // PORTDSetup
  wr_buf.Data[4]=PD_WORK;               // 0001 0000                    //  OE
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," read flash word high byte");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---  ,  PINB
  rd_buf=ReadFrom(1);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");         //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  //---              ---
  if(!Action)   //    
  {     SelectedChip.Area1[Address+1] = rd_buf.Data[0];
  }
  else          //    
  {     if(SelectedChip.Area1[Address+1] != rd_buf.Data[0])       //      ,   
        {       return 0;       //  ,   
        }
  }
  return true;
}
//---------------------------------------------------------------------------
//---         -----------------
//---------------------------------------------------------------------------
bool ATtiny2313_LoadProgramMemoryPage(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char b;      //     

  //if( (SelectedChip.Area1[Address]==0xFF) && (SelectedChip.Area1[Address+1]==0xFF) )
  //{     return true;    //    0xFF -  (      )
  //}

  b = Address>>1; //     (      ,    16 )
  if(Address == 0)
  {     if(!PhaseA(0x10))       return 0;       // Load Command 0x10 - Write Flash
  }
  if(!PhaseB(b))                                return 0;       // Load Address Low Byte
  if(!PhaseD(SelectedChip.Area1[Address]))      return 0;       // Load Data Low Byte
  if(!PhaseE(SelectedChip.Area1[Address+1]))    return 0;       // Load Data High Byte
  Sleep(Delays);//////////
  return true;
}
//---------------------------------------------------------------------------
//---          --------
//---------------------------------------------------------------------------
bool ATtiny2313_WriteProgramMemoryPage(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char a;              //    

  a = Address>>9;                       //    
  if(!PhaseC(a))        return 0;       // Load Address High Byte
  //--- PHASE G (Program Page) START ---
  //---     WR ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                          // Do_Extended_Command
  wr_buf.Data[1]=0x03;                                          // GiveWRNegPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," GiveWRNegPulse");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(Delays);//////////
  //   RDY = high
  if(!WaitRDYHigh)      return 0;       //    -   
  //--- PHASE G (Program Page) END ---
  //---     (  ) -    
  if( (Address+2) == SelectedChip.Area1Size)
  {     //--- PHASE I (End Page Programming) ---
        if(!PhaseA(0x0))        return 0;       // Load Command 0x0 - No Operation
        //--- PHASE I (End Page Programming) END ---
  }
  return true;
}
//---------------------------------------------------------------------------
/////////////////////////////
//    EEPROM //
/////////////////////////////
//---------------------------------------------------------------------------
//---    EEPROM    ---------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_ReadEEPROMMemory(unsigned int Address, bool Action)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char a,b;            //    EEPROM

  //        
  b = Address;          //    
  a = Address>>8;       //     (   128  EEPROM,     0)
  if(Address==0)
  {    if(!PhaseA(0x03))        return 0;       // Load Command 0x03 - Read EEPROM
  }
  if(!PhaseC(a))        return 0;       // Load Address High Byte
  if(!PhaseB(b))        return 0;       // Load Address Low Byte
  //--- set DDRB to input and OE to 0 to read a EEPROM byte (BS1 is already 0 after PhaseB)
  wr_buf.Size=7;
  wr_buf.Data[0]=0x0B;                                          // DDRBSetup
  wr_buf.Data[1]=0x0;                   // 0000 0000            //    
  wr_buf.Data[2]=0x15;                                          // PORTDSetup
  wr_buf.Data[3]=PD_WORK & ~(1<<OE);    // 0001 0000            //  OE
  wr_buf.Data[4]=0x17;                                          // PINBRead
  wr_buf.Data[5]=0x15;                                          // PORTDSetup
  wr_buf.Data[6]=PD_WORK;               // 0001 0100            //  OE
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," read EEPROM byte");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---  ,  PINB
  rd_buf=ReadFrom(1);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");         //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  //---               ---
  if(!Action)   //    
  {     SelectedChip.Area2[Address] = rd_buf.Data[0];
  }
  else          //    
  {     if(SelectedChip.Area2[Address] != rd_buf.Data[0])       //      ,   
        {       return 0;       //   
        }
  }
  return true;
}
//---------------------------------------------------------------------------
//---           
//---------------------------------------------------------------------------
bool ATtiny2313_WriteEEPROMMemory(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char a,b;      //        

  b = Address;          //    (  4 )
  a = Address>>8;       //    
  if(Address == 0)
  {     if(!PhaseA(0x11))                       return 0;       // Load Command 0x11 - Write EEPROM
  }
  if(Address%4 == 0)                                            //  EEPROM 4 ,     
  {     if(!PhaseC(a))                          return 0;       // Load Address High Byte
  }
  if(!PhaseB(b))                                return 0;       // Load Address Low Byte
  if(!PhaseD(SelectedChip.Area2[Address]))      return 0;       // Load Data Low Byte
  if((Address+1)%4 == 0)            //  EEPROM 4 ,     
  {     //--- PHASE K (Program EEPROM Page) START ---
        //---     WR (BS already 0 after PhaseD) ---
        wr_buf.Size=2;
        wr_buf.Data[0]=0x1D;                                            // Do_Extended_Command
        wr_buf.Data[1]=0x03;                                            // GiveWRNegPulse
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," GiveWRNegPulse");       //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        Sleep(Delays);//////////
        //   RDY = high
        if(!WaitRDYHigh)        return 0;       //    -   
        //--- PHASE K (Program EEPROM Page) END ---
  }
  return true;
}
//---------------------------------------------------------------------------
/////////////////////
// FUSES, LOCK ... //
/////////////////////
//---------------------------------------------------------------------------
//---  FUSE- & LOCK- ---------------------------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_ReadFuseLockBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," read fuse- & lock-bits");           //     Fuse- & Lock-
  if(!PhaseA(0x04))                       return 0;     // Load Command 0x04 - Read Fuse & Lock
  //--- set DDRB to input, OE to 0, BS2(XA1)=0, BS1=0 to read fuse low byte
  wr_buf.Size=7;
  wr_buf.Data[0]=0x0B;                                  // DDRBSetup
  wr_buf.Data[1]=0x0;   // 0000 0000                    //    
  wr_buf.Data[2]=0x11;                                  // PORTCSetup
  wr_buf.Data[3]=PC_WORK;               // 0000 1000    //  XA1,  BS1
  wr_buf.Data[4]=0x15;                                  // PORTDSetup
  wr_buf.Data[5]=PD_WORK & ~(1<<OE);    // 0001 0000    //  OE
  wr_buf.Data[6]=0x17;                                  // PINBRead
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," read Fuse Low bits"); //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---  ,  PINB
  rd_buf=ReadFrom(1);                              // 
  if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  SelectedChip.Area3[0] = rd_buf.Data[0];       //     Fuse Low
  //--- set BS2(XA1)=1, BS1=1 to read fuse high byte
  wr_buf.Size=3;
  wr_buf.Data[0]=0x11;                                                  // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1 << BS1) | (1<<XA1);       // 0001 1010    //  XA1,  BS1
  wr_buf.Data[2]=0x17;                                                  // PINBRead
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," read Fuse High bits");        //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---  ,  PINB
  rd_buf=ReadFrom(1);                              // 
  if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  SelectedChip.Area3[1] = rd_buf.Data[0];       //     Fuse High
  //--- set BS2(XA1)=1, BS1=0 to read fuse extended byte
  wr_buf.Size=3;
  wr_buf.Data[0]=0x11;                                                  // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1<<XA1);       // 0001 1000                 //  XA1,  BS1
  wr_buf.Data[2]=0x17;                                                  // PINBRead
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," read Fuse Extended bits");    //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---  ,  PINB
  rd_buf=ReadFrom(1);                              // 
  if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  SelectedChip.Area3[2] = rd_buf.Data[0];       //     Fuse Extended
  //--- set BS2(XA1)=0, BS1=1 to read Lock byte
  wr_buf.Size=5;
  wr_buf.Data[0]=0x11;                                          // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1<<BS1);    // 0001 1000            //  XA1,  BS1
  wr_buf.Data[2]=0x17;                                          // PINBRead
  wr_buf.Data[3]=0x15;                                          // PORTDSetup
  wr_buf.Data[4]=PD_WORK;               // 0001 0100            //  OE
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," read Lock bits");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---  ,  PINB
  rd_buf=ReadFrom(1);                              // 
  if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  SelectedChip.Area3[3] = rd_buf.Data[0];       //     Lock

  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---  LOCK- -----------------------------------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_WriteLockBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write lock-bits");    //     Lock-
  if(!PhaseA(0x20))                       return 0;     // Load Command 0x20 - Write Lock
  if(!PhaseD(SelectedChip.Area3[3]))      return 0;     // Load Data Low Byte = Lock Bits
  //---     WR ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                          // Do_Extended_Command
  wr_buf.Data[1]=0x03;                                          // GiveWRNegPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," GiveWRNegPulse");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(Delays);//////////
  //   RDY = high
  if(!WaitRDYHigh)        return 0;     //    -   
  SendMsgToLog(19," with success.");    //   
  return true;
}
//---------------------------------------------------------------------------
//---    -----------------------------------------------
//---------------------------------------------------------------------------
unsigned int ATtiny2313_ReadSignatureBytes()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned int ChipID, Temp;    //  

  SendMsgToLog(18," read signature bytes");     //      
  ChipID = 0;                                   //  
  //---   3   ---
  if(!PhaseA(0x08))                       return 0;     // Load Command 0x08 - Read Signature Bytes
  for(unsigned char i=0;i<3;i++)
  {     //---    ,    
        if(!PhaseB(i))                    return 0;     // Load Address Low
        //--- READ DATA FROM THE CHIP ---
        //---    B  ,  OE, BS1=0,    B,  OE
        wr_buf.Size=7;
        wr_buf.Data[0]=0x0B;                                            // DDRBSetup
        wr_buf.Data[1]=0x0;                                             //    
        wr_buf.Data[2]=0x15;                                            // PORTDSetup
        wr_buf.Data[3]=PD_WORK & ~(1<<OE);      // 0001 0000            //  OE (BS1 is already 0 after PhaseB)
        wr_buf.Data[4]=0x17;                                            // PINBRead
        wr_buf.Data[5]=0x15;                                            // PORTDSetup
        wr_buf.Data[6]=PD_WORK;                 // 0001 0100            //  OE
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," read Signature Byte");  //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        //  ,  PINB
        rd_buf=ReadFrom(1);                                   // 
        if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");    //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return 0;
                         }
        //---  ID 
        Temp = rd_buf.Data[0];
        ChipID += (Temp << (8*i));
  }
  SendMsgToLog(19,"");            //    
  return ChipID;
}
//---------------------------------------------------------------------------
//---  FUSE- (LOW FUSE BYTE) -------------------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_WriteFuseBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write fuse low byte");              //       
  if(!PhaseA(0x40))                       return 0;     // Load Command 0x40 - Write Fuse
  if(!PhaseD(SelectedChip.Area3[0]))      return 0;     // Load Data Low Byte = Fuse Low
  //    BS2(XA1) = 0, BS1 = 0
  // that's already done after PhaseD
  //---     WR ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                          // Do_Extended_Command
  wr_buf.Data[1]=0x03;                                          // GiveWRNegPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," GiveWRNegPulse");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(Delays);//////////
  //   RDY = high
  if(!WaitRDYHigh)        return 0;     //    -   
  SendMsgToLog(19," with success.");    //   
  return true;
}
//---------------------------------------------------------------------------
//---  FUSE- (HIGH FUSE BYTE) ------------------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_WriteFuseHighBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write fuse high byte");             //       
  if(!PhaseA(0x40))                       return 0;     // Load Command 0x40 - Write Fuse
  if(!PhaseD(SelectedChip.Area3[1]))      return 0;     // Load Data Low Byte = Fuse High
  //---     BS1=1, XA1(BS2)=0 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x11;                                                  // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1<<BS1);    // 0000 1010                    //  XA1(BS2),  BS1
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set BS1=1, BS2(XA1)=0");      //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     WR ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                  // Do_Extended_Command
  wr_buf.Data[1]=0x03;                                  // GiveWRNegPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x1D 0x03");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(Delays);//////////
  //   RDY = high
  if(!WaitRDYHigh)        return 0;     //    -   
  SendMsgToLog(19," with success.");    //   
  return true;
}
//---------------------------------------------------------------------------
//---  FUSE- (EXTENDED FUSE BYTE) --------------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_WriteFuseExtendedBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write fuse extended byte");         //       
  if(!PhaseA(0x40))                       return 0;     // Load Command 0x40 - Write Fuse
  if(!PhaseD(SelectedChip.Area3[2]))      return 0;     // Load Data Low Byte = Fuse High
  //---     BS1=0, XA1(BS2)=1 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x11;                                                  // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1<<XA1);    // 0001 1000                    //  XA1(BS2),  BS1
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set BS1=0, BS2(XA1)=1");      //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     WR ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                          // Do_Extended_Command
  wr_buf.Data[1]=0x03;                                          // GiveWRNegPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," GiveWRNegPulse");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(Delays);//////////
  //   RDY = high
  if(!WaitRDYHigh)        return 0;     //    -   
  SendMsgToLog(19," with success.");    //   
  return true;
}
//---------------------------------------------------------------------------
//---    ----------------------------------------------
//---------------------------------------------------------------------------
bool ATtiny2313_ReadCalibrationBytes()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," read calibration bytes");   //      
  if(!PhaseA(0x40))                       return 0;     // Load Command 0x40 - Write Fuse
  for(unsigned char i=0;i<SelectedChip.Area4Size;i++)
  {     //---    ,    
        if(!PhaseB(i))   return 0;      // Load Address Low Byte = i
        //---     BS1=0, XA1(BS2)=1 ---
        wr_buf.Size=2;
        wr_buf.Data[0]=0x11;                                                  // PORTCSetup
        wr_buf.Data[1]=PC_WORK | (1<<BS1);    // 0000 1010                    //  XA1(BS2),  BS1
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," set BS1=1, BS2(XA1)=0");      //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        //--- READ DATA FROM THE CHIP ---
        //---    B  ,  OE,    B,  OE
        wr_buf.Size=7;
        wr_buf.Data[0]=0x0B;                                                    // DDRBSetup
        wr_buf.Data[1]=0x0;                                                     //    
        wr_buf.Data[2]=0x15;                                                    // PORTDSetup
        wr_buf.Data[3]=PD_WORK & ~(1<<OE);      // 0001 0000                    //  OE
        wr_buf.Data[4]=0x17;                                                    // PINBRead
        wr_buf.Data[5]=0x15;                                                    // PORTDSetup
        wr_buf.Data[6]=PD_WORK;                 // 0001 0100                    //  OE
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," read Calibration Bytes");       //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        //  ,  PINB
        rd_buf=ReadFrom(1);                                   // 
        if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");    //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return 0;
                         }
        //---    
        SelectedChip.Area4[i] = rd_buf.Data[0];
  }
  SendMsgToLog(19,"");            //    
  return true;
}
//---------------------------------------------------------------------------

