//---------------------------------------------------------------------------
#pragma hdrstop

#include "ATtiny26.h"           //       ATtiny26
#include "PortFunctions.h"      //      
#include "LogFunctions.h"       //      
#include "ChipFunctions.h"      //     
#include "Main.h"               //      
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---       PROGRAMMING ENABLE -----
//---------------------------------------------------------------------------
bool ATtiny26_ProgrammingEnable()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," execute the sinchronization with chip");    //     
  //---      SS ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x48;                                    // SetSS_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x48");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---      SS ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x47;                                    // ClearSS_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x47");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //---   20  ---
  Sleep(20);
  //---       Programming Enable ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x53;
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x53 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-      ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---  ,    - ,   ,   -   
  if(rd_buf.Data[1]!=0x53) return false;                //    ( 1)
  SendMsgToLog(19," with success.");                    // ,    
  return true;
}
//---------------------------------------------------------------------------
//---    CHIP ERASE -------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_ChipErase()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," erase the chip");    //     
  //---       ChipErase ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x80;
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x80 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---      9  ---
  Sleep(9);
  //--- ,    ---
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
///////////////////////////////
//     //
///////////////////////////////
//---------------------------------------------------------------------------
//---      -------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_ReadProgramMemory(unsigned int Address, bool Action)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char H;      // ,      
  unsigned char a, b;    //      

  //         
  H =(Address&1)<<3;    //   ,        3- 
  b = Address>>1;       //    
  a = (Address>>9)&3;   //     (,  2   - )

  //---       ReadProgramMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=0x0;                           //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=a;
  wr_buf.Data[4]=0x20|H;                        //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 b a 0x2H");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---              ---
  if(!Action)   //    
  {     SelectedChip.Area1[Address] = rd_buf.Data[0];
  }
  else          //    
  {     if(SelectedChip.Area1[Address] != rd_buf.Data[0])       //      ,   
        {       return false;           //   
        }
  }
  return true;
}
//---------------------------------------------------------------------------
//---         -----------------
//---------------------------------------------------------------------------
bool ATtiny26_LoadProgramMemoryPage(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char H;      // ,      
  unsigned char b;      //     

  //         
  H =(Address&1)<<3;    //   ,        3- 
  b = (Address>>1)&0xF; //       (   ,    16 )

  //---       LoadProgramMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=SelectedChip.Area1[Address];   //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=0x0;
  wr_buf.Data[4]=0x40|H;                        //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 Data b 0x00 0x4H");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  return true;
}
//---------------------------------------------------------------------------
//---          --------
//---------------------------------------------------------------------------
bool ATtiny26_WriteProgramMemoryPage(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char a,b;      //   

  b = (Address>>1)&0xF0; //     (   ,     16)
  a = (Address>>9)&3;    //     (,  2   - )

  //---       LoadProgramMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=0x0;                           //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=a;
  wr_buf.Data[4]=0x4C;                          //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 b a 0x4C");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(5);
  return true;
}
//---------------------------------------------------------------------------
/////////////////////////////
//    EEPROM //
/////////////////////////////
//---------------------------------------------------------------------------
//---    EEPROM    ---------------------------
//---------------------------------------------------------------------------
bool ATtiny26_ReadEEPROMMemory(unsigned int Address, bool Action)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char b;      //   

  b = Address&0x7F;     //     (   128  EEPROM)

  //---       ReadEEPROMMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=0x0;                           //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=0x0;
  wr_buf.Data[4]=0xA0;                          //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 b 0x00 0xA0");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---               ---
  if(!Action)   //    
  {     SelectedChip.Area2[Address] = rd_buf.Data[0];
  }
  else          //    
  {     if(SelectedChip.Area2[Address] != rd_buf.Data[0])       //      ,   
        {       return false;           //   
        }
  }
  return true;
}
//---------------------------------------------------------------------------
//---     EEPROM    ---------------------
//---------------------------------------------------------------------------
bool ATtiny26_WriteEEPROMMemory(unsigned int Address)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned char b;      //  

  b = Address&0x7F;     //     (   128  EEPROM)

  //---       LoadProgramMemory ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                          // LoadPaket_SPI
  wr_buf.Data[1]=SelectedChip.Area2[Address];   //    
  wr_buf.Data[2]=b;
  wr_buf.Data[3]=0x0;
  wr_buf.Data[4]=0xC0;                          //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 Data b 0x00 0xC0");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(9);
  return true;
}
//---------------------------------------------------------------------------
/////////////////////
// FUSES, LOCK ... //
/////////////////////
//---------------------------------------------------------------------------
//---  LOCK- -----------------------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_ReadLockBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," read lock-bits");    //     Lock-
  //---       ReadLockBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x00;
  wr_buf.Data[4]=0x58;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x00 0x58");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---        ---
  SelectedChip.Area3[2] = rd_buf.Data[0]|0xFC;  //    2 
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---  LOCK- -----------------------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_WriteLockBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write lock-bits");    //     Lock-
  //---       WriteLockBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=SelectedChip.Area3[2]|0xFC;              //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0xE0;
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 Lock 0x00 0xE0 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(5);
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---    -----------------------------------------------
//---------------------------------------------------------------------------
unsigned int ATtiny26_ReadSignatureBytes()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned int ChipID, Temp;    //  

  SendMsgToLog(18," read signature bytes");     //      
  ChipID = 0;                                   //  
  //---       Read Signature Byte ---
  for(unsigned char i=0;i<3;i++)
  {     wr_buf.Size=5;
        wr_buf.Data[0]=0x44;                                 // LoadPaket_SPI
        wr_buf.Data[1]=0x0;
        wr_buf.Data[2]=i;                                    //   
        wr_buf.Data[3]=0x0;
        wr_buf.Data[4]=0x30;                                 //  Read Signature Byte
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 i 0x00 0x30");   //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        wr_buf.Size=1;
        wr_buf.Data[0]=0x46;                                  // StartChange_SPI
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                               CloseHandle(Port);
                                           Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        Sleep(Delays);//////////
        //---  SPI -  (        ) ---
        wr_buf.Size=1;
        wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
        //  ,  SPI - 
        rd_buf=ReadFrom(8);                                   // 
        if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return 0;
                         }
        //---  ID 
        Temp = rd_buf.Data[0];
        ChipID += (Temp << (8*i));
  }
  SendMsgToLog(19,"");            //    
  return ChipID;
}
//---------------------------------------------------------------------------
//---  FUSE- (LOW FUSE BYTE) -------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_WriteFuseBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write fuse low byte");              //       
  //---       WriteFuseBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=SelectedChip.Area3[0];                   //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0xA0;
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 Fuse High 0x00 0xA0 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(5);
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---  FUSE- (HIGH FUSE BYTE) ------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_WriteFuseHighBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," write fuse high byte");             //       
  //---       WriteFuseHighBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=SelectedChip.Area3[1]|0xE0;              //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0xA8;
  wr_buf.Data[4]=0xAC;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 Fuse High 0x00 0xA8 0xAC");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  Sleep(5);
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---  FUSE- (LOW FUSE BYTE) -------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_ReadFuseBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," read fuse low byte");       //     Low Fuse Byte
  //---       ReadFuseBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x00;
  wr_buf.Data[4]=0x50;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x00 0x50");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---        ---
  SelectedChip.Area3[0] = rd_buf.Data[0];
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---  FUSE- (HIGH FUSE BYTE) ------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_ReadFuseHighBits()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," read fuse high byte");       //     High Fuse Byte
  //---       ReadFuseHighBits ---
  wr_buf.Size=5;
  wr_buf.Data[0]=0x44;                                    // LoadPaket_SPI
  wr_buf.Data[1]=0x00;                                    //    
  wr_buf.Data[2]=0x00;
  wr_buf.Data[3]=0x08;
  wr_buf.Data[4]=0x58;                                    //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 0x00 0x08 0x58");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  wr_buf.Size=1;
  wr_buf.Data[0]=0x46;                                  // StartChange_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(Delays);//////////
  //---  SPI -  (     32-     ) ---
  wr_buf.Size=1;
  wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //  SPI -  (  32-  SPI    )
  rd_buf=ReadFrom(8);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return false;
                   }
  //---        ---
  SelectedChip.Area3[1] = rd_buf.Data[0]|0xE0;
  SendMsgToLog(19," with success.");            //   
  return true;
}
//---------------------------------------------------------------------------
//---    ----------------------------------------------
//---------------------------------------------------------------------------
bool ATtiny26_ReadCalibrationBytes()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," read calibration bytes");   //      
  //---       Read Calibration Byte ---
  for(unsigned char i=0;i<SelectedChip.Area4Size;i++)
  {     wr_buf.Size=5;
        wr_buf.Data[0]=0x44;                                 // LoadPaket_SPI
        wr_buf.Data[1]=0x0;
        wr_buf.Data[2]=i;                                    //   
        wr_buf.Data[3]=0x0;
        wr_buf.Data[4]=0x38;                                 //  Read Calibration Byte
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x44 0x00 i 0x00 0x38");   //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return false;
                             }
        wr_buf.Size=1;
        wr_buf.Data[0]=0x46;                                  // StartChange_SPI
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x46");       //   
                               CloseHandle(Port);
                                           Port=INVALID_HANDLE_VALUE;
                               return false;
                             }
        Sleep(Delays);//////////
        //---  SPI -  (        ) ---
        wr_buf.Size=1;
        wr_buf.Data[0]=0x45;                                  // ReadPaketSPI
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x45");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
        //  ,  SPI - 
        rd_buf=ReadFrom(8);                                   // 
        if(!rd_buf.Size) { SendMsgToLog(7," (SPI paket)");    //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return false;
                         }
        //---    
        SelectedChip.Area4[i] = rd_buf.Data[0];
  }
  SendMsgToLog(19,"");            //    
  return true;
}
//---------------------------------------------------------------------------
