//---------------------------------------------------------------------------
#pragma hdrstop

#include "AVRFunctions.h"       //       AVR
#include "PortFunctions.h"      //      
#include "LogFunctions.h"       //      
#include "ChipFunctions.h"      //     
#include "Main.h"               //      
//---------------------------------------------------------------------------
#pragma package(smart_init)
////////////////////////////////////////////////////////////
//      ,     //
////////////////////////////////////////////////////////////
//***************************************************************************
//***   () **********************************************
//***************************************************************************
//---------------------------------------------------------------------------
//---     RDY ---------------------------------------
//---------------------------------------------------------------------------
bool WaitRDYHigh()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  for(unsigned char i=0;i<CYCLES;i++)   // CYCLES -    AVRFunctions
  {     //---   PINC
        wr_buf.Size=1;
        wr_buf.Data[0]=0x18;                                  // PINCRead
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," PINCRead");   //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        //---  ,  PINC
        rd_buf=ReadFrom(1);                                   // 
        if(!rd_buf.Size) { SendMsgToLog(7," (PINC)");         //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return 0;
                         }
        if((rd_buf.Data[0]&0x01))     return true;            //    (RDY = 1) - , 
        Sleep(PAUSE);                   // PAUSE -    AVRFunctions
  }
  return false;         //        RDY
}
//---------------------------------------------------------------------------
//--- PHASE A (Load Command) ------------------------------------------------
//---------------------------------------------------------------------------
bool PhaseA(unsigned char Command)
{ WRITE_BUFFER wr_buf;          //    

  //--- set XA1 (PC4), XA0 (PD6) to 10; BS1 (PC1), BS2 (PD7) to 00 ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                                    // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1 << XA1);  // 0001 1000      //  XA1,  BS1
  wr_buf.Data[2]=0x15;                                    // PORTDSetup
  wr_buf.Data[3]=PD_WORK;               // 0001 0100      //  XA0,  BS2
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set XA1, XA0 = 10, BS1, BS2 = 00");        //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---   Command ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0B;                                          // DDRBSetup
  wr_buf.Data[1]=0xFF;          // 1111 1111                    //    
  wr_buf.Data[2]=0x0D;                                          // PORTBSetup
  wr_buf.Data[3]=Command;                                       // Command
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set Command");        //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     XTAL1 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                          // Do_Extended_Command
  wr_buf.Data[1]=0x02;                                          // GiveXtalPosPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," GiveXtalPosPulse");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  return true;
}
//---------------------------------------------------------------------------
//--- PHASE B (Load Address Low Byte) ---------------------------------------
//---------------------------------------------------------------------------
bool PhaseB(unsigned char Address)
{ WRITE_BUFFER wr_buf;          //    

  //--- set XA1 (PC4), XA0 (PD6) to 00; BS1 (PC1), BS2 (PD7) to 00 ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                                                          // PORTCSetup
  wr_buf.Data[1]=PC_WORK; // 0000 1000                                          //  XA1,  BS1
  wr_buf.Data[2]=0x15;                                                          // PORTDSetup
  wr_buf.Data[3]=PD_WORK; // 0001 0100                                          //  XA0,  BS2
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set XA1, XA0 = 00, BS1, BS2 = 00");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     (i) ---
  wr_buf.Size=4;                                                        //   B  
  wr_buf.Data[0]=0x0B;                                                  // DDRBSetup
  wr_buf.Data[1]=0xFF;    // 1111 1111                                  //    
  wr_buf.Data[2]=0x0D;                                                  // PORTBSetup
  wr_buf.Data[3]=Address;                                               //   
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," Load Address Low Byte");      //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     XTAL1 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                  // Do_Extended_Command
  wr_buf.Data[1]=0x02;                                  // GiveXtalPosPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x1D 0x02");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  return true;
}
//---------------------------------------------------------------------------
//--- PHASE C (Load Address High Byte) --------------------------------------
//---------------------------------------------------------------------------
bool PhaseC(unsigned char Address)
{ WRITE_BUFFER wr_buf;          //    

  //--- set XA1 (PC4), XA0 (PD6) to 00; BS1 (PC1), BS2 (PD7) to 10 ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                                                          // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1 << BS1);  // 0000 1010                            //  XA1,  BS1
  wr_buf.Data[2]=0x15;                                                          // PORTDSetup
  wr_buf.Data[3]=PD_WORK;               // 0001 0100                            //  XA0,  BS2
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set XA1, XA0 = 00, BS1, BS2 = 10");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---      ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0B;                                    // DDRBSetup (   )
  wr_buf.Data[1]=0xFF;  // 1111 1111                      //    
  wr_buf.Data[2]=0x0D;                                    // PORTBSetup
  wr_buf.Data[3]=Address;                                 //    
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x0D a");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     XTAL1 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                  // Do_Extended_Command
  wr_buf.Data[1]=0x02;                                  // GiveXtalPosPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x1D 0x02");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  return true;
}
//---------------------------------------------------------------------------
//--- PHASE D (Load Data Low Byte) ------------------------------------------
//---------------------------------------------------------------------------
bool PhaseD(unsigned char Data)
{ WRITE_BUFFER wr_buf;          //    

  //--- set XA1 (PC4), XA0 (PD6) to 01; BS1 (PC1), BS2 (PD7) to 00 ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                                                          // PORTCSetup
  wr_buf.Data[1]=PC_WORK;               // 0000 1000                            //  XA1,  BS1
  wr_buf.Data[2]=0x15;                                                          // PORTDSetup
  wr_buf.Data[3]=PD_WORK | (1<<XA0);    // 0101 0100                            //  XA0,  BS2
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set XA1, XA0 = 01, BS1, BS2 = 00");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---      ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0B;                                                  // DDRBSetup
  wr_buf.Data[1]=0xFF;  // 1111 1111                                    //    
  wr_buf.Data[2]=0x0D;                                                  // PORTBSetup
  wr_buf.Data[3]=Data;                                                  //    
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," Load Data Low Byte");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     XTAL1 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                  // Do_Extended_Command
  wr_buf.Data[1]=0x02;                                  // GiveXtalPosPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x1D 0x02");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  return true;
}
//---------------------------------------------------------------------------
//--- PHASE E (Load Data High Byte) -----------------------------------------
//---------------------------------------------------------------------------
bool PhaseE(unsigned char Data)
{ WRITE_BUFFER wr_buf;          //    

  //--- set XA1 (PC4), XA0 (PD6) to 01; BS1 (PC1), BS2 (PD7) to 10 ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                                                          // PORTCSetup
  wr_buf.Data[1]=PC_WORK | (1<<BS1);    // 0000 1010                            //  XA1,  BS1
  wr_buf.Data[2]=0x15;                                                          // PORTDSetup
  wr_buf.Data[3]=PD_WORK | (1<<XA0);    // 0101 0100                            //  XA0
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set XA1, XA0 = 01, BS1, BS2 = 10");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---      ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0B;                                                  // DDRBSetup (   )
  wr_buf.Data[1]=0xFF;          // 1111 1111                            //    
  wr_buf.Data[2]=0x0D;                                                  // PORTBSetup
  wr_buf.Data[3]=Data;                                                  //    
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," Load Data High Byte");        //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     XTAL1 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                  // Do_Extended_Command
  wr_buf.Data[1]=0x02;                                  // GiveXtalPosPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x1D 0x02");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  return true;
}
//---------------------------------------------------------------------------
//--- PHASE F (Load Address Extended Byte) ----------------------------------
//---------------------------------------------------------------------------
bool PhaseF(unsigned char Address)
{ WRITE_BUFFER wr_buf;          //    

  //--- set XA1 (PC4), XA0 (PD6) to 00; BS1 (PC1), BS2 (PD7) to 01 ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                                                          // PORTCSetup
  wr_buf.Data[1]=PC_WORK;               // 0000 1000                            //  XA1,  BS1
  wr_buf.Data[2]=0x15;                                                          // PORTDSetup
  wr_buf.Data[3]=PD_WORK | (1 << BS2);  // 1001 0100                            //  XA0,  BS2
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set XA1, XA0 = 00, BS1, BS2 = 01");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---      ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0B;                                    // DDRBSetup (   )
  wr_buf.Data[1]=0xFF;  // 1111 1111                      //    
  wr_buf.Data[2]=0x0D;                                    // PORTBSetup
  wr_buf.Data[3]=Address;                                 //    
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x0D a");       //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  //---     XTAL1 ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                  // Do_Extended_Command
  wr_buf.Data[1]=0x02;                                  // GiveXtalPosPulse
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x1D 0x02");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  return true;
}
//***************************************************************************
//***   (  ) *******************************
//***************************************************************************
//---------------------------------------------------------------------------
//---      ---------------------------------
//---------------------------------------------------------------------------
bool EnterProgrammingMode()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," enter Programming Mode");    //        
  //---          ---
  //---  RDY       
  //---        (  ,   )
  //---      

  //---   B
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0B;                                    // DDRBSetup
  wr_buf.Data[1]=0x0;                                     //    
  wr_buf.Data[2]=0x0D;                                    // PORTBSetup
  wr_buf.Data[3]=0x0;                                     //    
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x0B 0x0 0x0D 0x0");        //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //---   C
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0F;                                    // DDRCSetup
  wr_buf.Data[1]=0x32;  // 0011 0010                      //     ,  RDY   
  wr_buf.Data[2]=0x11;                                    // PORTCSetup
  wr_buf.Data[3]=0x0;                                     //     (  )
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x0F 0x32 0x11 0x0");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  //---   D (     Prog_enable)
  //---   PG   ,        
  wr_buf.Size=4;
  wr_buf.Data[0]=0x13;                                    // DDRDSetup
  wr_buf.Data[1]=0xFC;  // 1111 1100                      //     
  wr_buf.Data[2]=0x15;                                    // PORTDSetup
  wr_buf.Data[3]=0x0;                                     //     (       WR, OE)
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x13 0xFC 0x15 0x0");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //---          AVR ---
  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;                                    //   
  wr_buf.Data[1]=0x0;                                     //  Enter_HVP_ProgMode_AVR
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x1D 0x0");     //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //---   D -     WR, OE (      )
  wr_buf.Size=2;
  wr_buf.Data[0]=0x15;                                    // PORTDSetup
  wr_buf.Data[1]=PD_WORK;       // 0001 0100              //  WR, OE  1
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x15 0x14(PD_WORK)"); //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  SendMsgToLog(19," with success.");                    // ,    
  return true;
}
//---------------------------------------------------------------------------
//---     --------------------------------------
//---------------------------------------------------------------------------
bool ExitProgrammingMode()
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    

  SendMsgToLog(18," exit Programming Mode");    //        
  //---      +5  +12  ---
  //--- +12      ,  +5  -    
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                                    // PORTCSetup
  wr_buf.Data[1]=0x0;                                     //     ( +12,   )
  wr_buf.Data[2]=0x0F;                                    // DDRCSetup
  wr_buf.Data[3]=0x32;  // 0011 0010                      //      (     ,  RDY)
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," 0x11 0x0 0x0F 0x32");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return false;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  SendMsgToLog(19," with success.");                    // ,    
  return true;
}
//---------------------------------------------------------------------------
//---    -----------------------------------------------
//---------------------------------------------------------------------------
unsigned int ReadSignatureBytes()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int ChipID, Temp;            //  

  SendMsgToLog(18," read signature bytes");     //      
  ChipID = 0;                                   //  

  if(!PhaseA(0x08))        return 0;            // Load Command 0x08 - Read Signaure Byte
  //---   3   ---
  for(unsigned char i=0;i<3;i++)
  {     if(!PhaseB(i))  return 0;               // Load Address Low Byte
        //--- READ DATA FROM THE CHIP ---
        //---    B  ,  BS=0,  OE,    B,  OE
        wr_buf.Size=7;
        wr_buf.Data[0]=0x0B;                                    // DDRBSetup
        wr_buf.Data[1]=0x0;                                     //    
        // BS is already 0 after PhaseB
        wr_buf.Data[2]=0x15;                                    // PORTDSetup
        wr_buf.Data[3]=PD_WORK & ~(1<<OE);      // 0001 0000    //  OE
        wr_buf.Data[4]=0x17;                                    // PINBRead
        wr_buf.Data[5]=0x15;                                    // PORTDSetup
        wr_buf.Data[6]=PD_WORK;                 // 0001 0100    //  OE
        if(!WriteTo(wr_buf)) { SendMsgToLog(6," Read Signature Bytes");  //   
                               CloseHandle(Port);
                               Port=INVALID_HANDLE_VALUE;
                               return 0;
                             }
        //  ,  PINB
        rd_buf=ReadFrom(1);                              // 
        if(!rd_buf.Size) { SendMsgToLog(7," (PINB)");    //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return 0;
                         }
        //---  ID 
        Temp = rd_buf.Data[0];
        ChipID += (Temp << (8*i));
  }
  SendMsgToLog(19,"");            //    
  return ChipID;
}
//---------------------------------------------------------------------------
